# from langchain_community.llms import OpenAI
# if not os.environ.get("OPENAI_API_KEY"):
#   os.environ["OPENAI_API_KEY"] = "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"
# if not os.environ.get("GEMINI_API_KEY"):
#     os.environ["GEMINI_API_KEY"] = "AIzaSyDzj2f8kaoEBFbjY3T0UTtnwTlHRwtvR1Y"
# from langchain.chat_models import init_chat_model
#
# # llm = init_chat_model("gpt-3.5-turbo-instruct", model_provider="openai",api_key="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")
# llm = init_chat_model("gemini-2.0-flash-exp", model_provider="google_vertexai",api_key="AIzaSyDzj2f8kaoEBFbjY3T0UTtnwTlHRwtvR1Y")
# json_schema = {
#     "title": "joke",
#     "description": "Joke to tell user.",
#     "type": "object",
#     "properties": {
#         "setup": {
#             "type": "string",
#             "description": "The setup of the joke",
#         },
#         "punchline": {
#             "type": "string",
#             "description": "The punchline to the joke",
#         },
#         "rating": {
#             "type": "integer",
#             "description": "How funny the joke is, from 1 to 10",
#             "default": None,
#         },
#     },
#     "required": ["setup", "punchline"],
# }
# structured_llm = llm.with_structured_output(json_schema)
#
# structured_llm.invoke("Tell me a joke about cats")


# from langchain_google_genai import ChatGoogleGenerativeAI
# from langchain_core.runnables import RunnableGenerator
# from typing import Iterator

# if not os.environ.get("GOOGLE_API_KEY"):
#   os.environ["GOOGLE_API_KEY"] = "AIzaSyDzj2f8kaoEBFbjY3T0UTtnwTlHRwtvR1Y"

# def _generate(input: Iterator) -> Iterator[str]:
#     print(1111)
#     yield from "foo bar"
#
# llm = ChatGoogleGenerativeAI(
#     model="gemini-2.0-flash-exp",
#
#     google_api_key="AIzaSyDzj2f8kaoEBFbjY3T0UTtnwTlHRwtvR1Y"
# ).with_fallbacks([RunnableGenerator(_generate)])
# print(llm.invoke("给我讲一个笑话"))


from jackchendongtest.observer import  LangChainObserver, ObserverConfig, SessionConfig,TracerConfig
from langchain_google_genai import ChatGoogleGenerativeAI

ObserverConfig = ObserverConfig(projectId="4723b7100da6a05a4c68fe43c0f7a83f", threadId="test",
                                apiHost="http://39.104.13.226:8088", apiKey="osk-JnSwDBQakTZeNiMYn1UknmArIHzNUxiY")
client = LangChainObserver(ObserverConfig)
client.startSession(SessionConfig(sessionName="eat"))
tracer = client.createTracer(TracerConfig(agentName="agent"))
mode = ChatGoogleGenerativeAI(model="gemini-2.0-flash-exp", google_api_key="AIzaSyDzj2f8kaoEBFbjY3T0UTtnwTlHRwtvR1Y")
res = mode.invoke("给我讲一个笑话", {"callbacks": [tracer]})

print(res)




from typing import Any, Dict
from uuid import UUID
from datetime import datetime

from langchain_core.messages import HumanMessage, AIMessage
from langchain_core.outputs import LLMResult, ChatGeneration
from pydantic.v1.typing import NoneType


def recursive_to_dict(obj: Any) -> Dict:
    if isinstance(obj, dict):
        return {key: recursive_to_dict(value) for key, value in obj.items()}
    elif isinstance(obj, list):
        return [recursive_to_dict(item) for item in obj]
    elif isinstance(obj, UUID):
        return str(obj)  # 将 UUID 转换为字符串
    elif isinstance(obj, int):
        return obj  # 将 datetime 转换为 ISO 格式字符串
    elif isinstance(obj, LLMResult):
        return obj.model_dump()
        # return obj.__dict__
    elif isinstance(obj, HumanMessage):
        return obj.model_dump()
    elif isinstance(obj, ChatGeneration):
        return obj.model_dump()
    elif isinstance(obj, AIMessage):
        return obj.model_dump()
    elif isinstance(obj, NoneType):
        return None

    return obj



import time
from typing import Optional, Dict, Any, List

from langchain_core.agents import AgentAction, AgentFinish
from langchain_core.callbacks import BaseCallbackHandler
from langchain_core.outputs import LLMResult
from langchain_core.messages import BaseMessage
from pydantic import BaseModel

from jackchendongtest.config import API_KEY, API_HOST, OUTPUT_LOG_SWITCH
from jackchendongtest.event import EventManager, Event, EventType
from jackchendongtest.server import Server, Session, Agent, Thread


class ObserverConfig:
    def __init__(self, projectId: str, apiKey: Optional[str] = None, apiHost: Optional[str] = None,
                 threadId: Optional[str] = None):
        self.projectId = projectId
        self.apiKey = apiKey
        self.apiHost = apiHost
        self.threadId = threadId


class SessionConfig:
    def __init__(self, sessionName: str, externalUserId: Optional[str] = None):
        self.sessionName = sessionName
        self.externalUserId = externalUserId


class TracerConfig:
    def __init__(self, agentName: Optional[str]):
        self.agentName = agentName


class LangChainObserver:
    def __init__(self, observerConfig: ObserverConfig):
        self.observaSession = None
        self.observaAgent = None
        self.observaThread = None
        self.projectId = observerConfig.projectId
        apiKey = observerConfig.apiKey or API_KEY
        host = observerConfig.apiHost or API_HOST

        if observerConfig.threadId:
            self.observaThread = {'threadId': observerConfig.threadId}
        self.apiServer = Server(apiKey, self.projectId, host)

    def createTracer(self, traceConfig: Optional[TracerConfig]):
        self.createThread()
        if traceConfig and traceConfig.agentName:
            self.observaAgent = {
                "agentName": traceConfig.agentName
            }
            self.createAgent()
        return TracerObserver(self)

    def createThread(self):
        if self.observaThread and not self.observaThread.get("observaThreadId"):
            self.observaThread['observaThreadId'] = self.apiServer.createThread(
                Thread(self.observaThread.get("threadId")))

    def createAgent(self):
        if self.observaAgent and not self.observaAgent.get('id'):
            self.observaAgent['id'] = self.apiServer.createAgent(Agent(self.observaAgent.get('agentName') or ""))

    def startSession(self, sessionConfig: SessionConfig):
        session = Session(sessionConfig.sessionName, sessionConfig.externalUserId)
        sessionId = self.apiServer.createSession(session)
        self.observaSession = {
            "sessionName": sessionConfig.sessionName,
            "externalUserId": sessionConfig.externalUserId,
            "id": sessionId
        }
        return self

    def getProjectId(self):
        return self.projectId


class TracerObserver(BaseCallbackHandler):

    def __init__(self, client: LangChainObserver):
        super()
        self.name = "TracerObserver"
        self.outputLogSwitch = OUTPUT_LOG_SWITCH
        self.client = client
        self.eventManager = EventManager(self.client.projectId)
        self.observaAgent = None
        self.observaThread = None

    def outputLog(self, fName: str, data: Any):
        if self.outputLogSwitch:
            print(fName, data)

    def getAgentId(self):
        return (self.observaAgent and self.observaAgent.get('id')) or self.client.observaAgent.get('id', '')

    def getThreadId(self):
        return (self.observaThread and self.observaThread.id) or self.client.observaThread.get('observaThreadId', '')

    def getSessionId(self):
        return self.client.observaSession.get('id', '')

    def createThread(self, threadId: str):
        if not self.observaThread:
            self.observaThread = {
                "threadId": threadId
            }
        if self.observaThread and not self.observaThread.threadId:
            self.observaThread.observaThreadId = self.client.apiServer.createThread(
                Thread(self.observaThread.threadId or ''))

    def createAgent(self, agentName: str):
        if not self.observaAgent:
            self.observaAgent = {
                "agentName": agentName
            }
        if self.observaAgent and not self.observaAgent.get('id'):
            self.observaAgent["id"] = self.client.apiServer.createAgent(Agent(agentName or ""))

    def on_chat_model_start(
            self, serialized: Dict[str, Any], messages: List[List[BaseMessage]], **kwargs
    ) -> None:
        self.outputLog("on_chat_model_start", {**kwargs, "serialized": serialized, "messages": messages})
        metadata = kwargs.get("metadata")
        if metadata and metadata.get('agentName'):
            self.createAgent(metadata.get('agentName'))
        if metadata and metadata.get('threadId'):
            self.createThread(metadata.get('threadId'))
        extraParams = kwargs.get('extra_params', {})
        self.eventManager.createEventAndTrace(Event(
            eventId=kwargs.get("run_id") and str(kwargs.get("run_id")),
            parentId=kwargs.get("parent_run_id") and str(kwargs.get("parent_run_id")),
            eventType=EventType.LLM,
            eventName=serialized.get('id')[len(serialized.get('id')) - 1],
            metadata=metadata,
            tags=kwargs.get('tags'),
            input=messages,
            startTime=int(time.time() * 1000),
            extraParams=extraParams,
            model=extraParams.get('invocation_params', {}).get('model') or \
                  extraParams.get('invocation_params', {}).get('modelName') or \
                  metadata.get('ls_model_name')
        ))

    def on_llm_start(
            self, serialized: Dict[str, Any], prompts: List[str], **kwargs: Any
    ) -> Any:
        self.outputLog("on_llm_start", {**kwargs, "serialized": serialized, "prompts": prompts})
        metadata = kwargs.get("metadata")
        if metadata and metadata.get('agentName'):
            self.createAgent(metadata.get('agentName'))
        if metadata and metadata.get('threadId'):
            self.createThread(metadata.get('threadId'))
        extraParams = kwargs.get('extra_params', {})
        self.eventManager.createEventAndTrace(Event(
            eventId=kwargs.get("run_id") and str(kwargs.get("run_id")),
            parentId=kwargs.get("parent_run_id") and str(kwargs.get("parent_run_id")),
            eventType=EventType.LLM,
            eventName=serialized.get('id')[len(serialized.get('id')) - 1],
            metadata=metadata,
            tags=kwargs.get('tags'),
            input=prompts,
            startTime=int(time.time() * 1000),
            extraParams=extraParams,
            model=extraParams.get('invocation_params', {}).get('model') or \
                  extraParams.get('invocation_params', {}).get('modelName') or \
                  metadata.get('ls_model_name')
        ))

    def on_llm_end(self, response: LLMResult, **kwargs) -> None:
        self.outputLog("on_llm_end", {**kwargs, "response": response})
        endTime = int(time.time() * 1000)
        event = self.eventManager.getEventById(str(kwargs.get('run_id')))
        if event:
            event.endTime = endTime
            if event.startTime:
                event.duration = endTime - event.startTime
            event.output = response
            event.tokenCount = event.output and event.output.llm_output and event.output.llm_output.get(
                'token_usage') and event.output.llm_output.get('token_usage').get("total_tokens") or 0
        if event and not event.parentId:
            self.client.apiServer.createEvent({
                "sessionId": self.getSessionId(),
                "threadId": self.getThreadId(),
                "agentId": self.getAgentId(),
                "logList": self.eventManager.getRootEvents()
            })

    def on_llm_error(
            self,
            error: BaseException,
            **kwargs: Any,
    ) -> Any:
        self.outputLog("on_llm_error", {**kwargs, "error": error})
        endTime = int(time.time() * 1000)
        event = self.eventManager.getEventById(str(kwargs.get('run_id')))
        if event:
            event.endTime = endTime
            if event.startTime:
                event.duration = endTime - event.startTime
            event.error = error
        if event and not event.parentId:
            self.client.apiServer.createEvent({
                "sessionId": self.getSessionId(),
                "threadId": self.getThreadId(),
                "agentId": self.getAgentId(),
                "logList": self.eventManager.getRootEvents()
            })

    def on_chain_start(
            self, serialized: Dict[str, Any], inputs: Dict[str, Any], **kwargs
    ) -> None:
        self.outputLog("on_chain_start", {**kwargs, "serialized": serialized, "inputs": inputs})
        metadata = kwargs.get("metadata")
        if metadata and metadata.get('agentName'):
            self.createAgent(metadata.get('agentName'))
        if metadata and metadata.get('threadId'):
            self.createThread(metadata.get('threadId'))

        self.eventManager.createEventAndTrace(Event(
            eventId=kwargs.get("run_id") and str(kwargs.get("run_id")),
            parentId=kwargs.get("parent_run_id") and str(kwargs.get("parent_run_id")),
            eventType=EventType.CHAIN,
            eventName=serialized.get('id')[len(serialized.get('id')) - 1],
            tags=kwargs.get('tags'),
            input=inputs,
            startTime=int(time.time() * 1000),
        ))

    def on_chain_end(self, outputs: Dict[str, Any], **kwargs) -> None:
        self.outputLog("on_chain_end", {**kwargs, "outputs": outputs})

        endTime = int(time.time() * 1000)
        event = self.eventManager.getEventById(kwargs.get('run_id'))
        if event:
            event.endTime = endTime
            if event.startTime:
                event.duration = endTime - event.startTime
            event.output = outputs

        if event and not event.parentId:
            self.client.apiServer.createEvent({
                "sessionId": self.getSessionId(),
                "threadId": self.getThreadId(),
                "agentId": self.getAgentId(),
                "logList": self.eventManager.getRootEvents()
            })

    def on_chain_error(
            self,
            error: BaseException,
            **kwargs: Any,
    ) -> Any:
        self.outputLog("on_chain_error", {**kwargs, "error": error})
        endTime = int(time.time() * 1000)
        event = self.eventManager.getEventById(str(kwargs.get('run_id')))
        if event:
            event.endTime = endTime
            if event.startTime:
                event.duration = endTime - event.startTime
            event.error = error

        if event and not event.parentId:
            self.client.apiServer.createEvent({
                "sessionId": self.getSessionId(),
                "threadId": self.getThreadId(),
                "agentId": self.getAgentId(),
                "logList": self.eventManager.getRootEvents()
            })

    def on_agent_action(
            self,
            action: AgentAction,
            **kwargs: Any,
    ) -> Any:
        self.outputLog("on_agent_action", {**kwargs, "action": action})
        event = self.eventManager.createEventAndTrace(Event(
            eventId=kwargs.get("run_id") and str(kwargs.get("run_id")),
            parentId=kwargs.get("parent_run_id") and str(kwargs.get("parent_run_id")),
            eventType=EventType.TOOL,
            eventName="Agent Action",
            tags=kwargs.get('tags'),
            input=action.tool_input,
            startTime=int(time.time() * 1000),
        ))
        if event and not event.parentId:
            self.client.apiServer.createEvent({
                "sessionId": self.getSessionId(),
                "threadId": self.getThreadId(),
                "agentId": self.getAgentId(),
                "logList": self.eventManager.getRootEvents()
            })

    def on_agent_finish(
            self,
            finish: AgentFinish,
            **kwargs: Any,
    ) -> Any:
        self.outputLog("on_agent_finish", {**kwargs, "finish": finish})
        endTime = int(time.time() * 1000)
        event = self.eventManager.getEventById(str(kwargs.get('run_id')))
        if event:
            event.endTime = endTime
            if event.startTime:
                event.duration = endTime - event.startTime
            event.output = finish.return_values
            if event and not event.parentId:
                self.client.apiServer.createEvent({
                    "sessionId": self.getSessionId(),
                    "threadId": self.getThreadId(),
                    "agentId": self.getAgentId(),
                    "logList": self.eventManager.getRootEvents()
                })
        else:
            event = self.eventManager.createEventAndTrace(Event(
                eventId=kwargs.get("run_id") and str(kwargs.get("run_id")),
                parentId=kwargs.get("parent_run_id") and str(kwargs.get("parent_run_id")),
                eventType=EventType.TOOL,
                eventName="Agent Finish",
                tags=kwargs.get('tags'),
                output=finish.return_values,
                startTime=endTime,
                endTime=endTime
            ))
            if event and not event.parentId:
                self.client.apiServer.createEvent({
                    "sessionId": self.getSessionId(),
                    "threadId": self.getThreadId(),
                    "agentId": self.getAgentId(),
                    "logList": self.eventManager.getRootEvents()
                })

    def on_tool_start(
            self,
            serialized: dict[str, Any],
            input_str: str,
            **kwargs: Any,
    ) -> Any:
        self.outputLog("on_tool_start", {**kwargs, "serialized": serialized, "input_str": input_str})
        metadata = kwargs.get("metadata")
        if metadata and metadata.get('agentName'):
            self.createAgent(metadata.get('agentName'))
        if metadata and metadata.get('threadId'):
            self.createThread(metadata.get('threadId'))
        self.eventManager.createEventAndTrace(Event(
            eventId=kwargs.get("run_id") and str(kwargs.get("run_id")),
            parentId=kwargs.get("parent_run_id") and str(kwargs.get("parent_run_id")),
            eventType=EventType.TOOL,
            eventName=serialized.get('id')[len(serialized.get('id')) - 1],
            tags=kwargs.get('tags'),
            input=input_str,
            startTime=int(time.time() * 1000),
        ))

    def on_tool_end(
            self,
            output: Any,
            **kwargs: Any,
    ) -> Any:
        self.outputLog("on_tool_end", {**kwargs, "output": output})
        endTime = int(time.time() * 1000)
        event = self.eventManager.getEventById(str(kwargs.get('run_id')))
        if event:
            event.endTime = endTime
            if event.startTime:
                event.duration = endTime - event.startTime
            event.output = output
        if event and not event.parentId:
            self.client.apiServer.createEvent({
                "sessionId": self.getSessionId(),
                "threadId": self.getThreadId(),
                "agentId": self.getAgentId(),
                "logList": self.eventManager.getRootEvents()
            })

    def on_tool_error(
            self,
            error: BaseException,
            **kwargs: Any,
    ) -> Any:
        self.outputLog("on_tool_error", {**kwargs, "error": error})
        endTime = int(time.time() * 1000)
        event = self.eventManager.getEventById(str(kwargs.get('run_id')))
        if event:
            event.endTime = endTime
            if event.startTime:
                event.duration = endTime - event.startTime
            event.error = error

        if event and not event.parentId:
            self.client.apiServer.createEvent({
                "sessionId": self.getSessionId(),
                "threadId": self.getThreadId(),
                "agentId": self.getAgentId(),
                "logList": self.eventManager.getRootEvents()
            })

    def on_retriever_start(
            self,
            serialized: dict[str, Any],
            query: str,
            **kwargs: Any,
    ) -> Any:
        self.outputLog("on_retriever_start", {**kwargs, "query": query, "serialized": serialized})
        metadata = kwargs.get("metadata")
        if metadata and metadata.get('agentName'):
            self.createAgent(metadata.get('agentName'))
        if metadata and metadata.get('threadId'):
            self.createThread(metadata.get('threadId'))
        self.eventManager.createEventAndTrace(Event(
            eventId=kwargs.get("run_id") and str(kwargs.get("run_id")),
            parentId=kwargs.get("parent_run_id") and str(kwargs.get("parent_run_id")),
            eventType=EventType.RETRIEVAL,
            eventName=serialized.get('id')[len(serialized.get('id')) - 1],
            tags=kwargs.get('tags'),
            input=query,
            startTime=int(time.time() * 1000),
        ))

    def on_retriever_end(
            self,
            documents: Any,
            **kwargs: Any,
    ) -> Any:
        self.outputLog("on_retriever_end", {**kwargs, "documents": documents})
        endTime = int(time.time() * 1000)
        event = self.eventManager.getEventById(str(kwargs.get('run_id')))
        if event:
            event.endTime = endTime
            if event.startTime:
                event.duration = endTime - event.startTime
            event.output = documents

        if event and not event.parentId:
            self.client.apiServer.createEvent({
                "sessionId": self.getSessionId(),
                "threadId": self.getThreadId(),
                "agentId": self.getAgentId(),
                "logList": self.eventManager.getRootEvents()
            })

    def on_retriever_error(
            self,
            error: BaseException,
            **kwargs: Any,
    ) -> Any:
        self.outputLog("on_retriever_error", {**kwargs, "error": error})
        endTime = int(time.time() * 1000)
        event = self.eventManager.getEventById(str(kwargs.get('run_id')))
        if event:
            event.endTime = endTime
            if event.startTime:
                event.duration = endTime - event.startTime
            event.error = error
        if event and not event.parentId:
            self.client.apiServer.createEvent({
                "sessionId": self.getSessionId(),
                "threadId": self.getThreadId(),
                "agentId": self.getAgentId(),
                "logList": self.eventManager.getRootEvents()
            })



import requests
from typing import Optional, Dict, Any, List
from urllib.parse import urljoin


class Thread:
    def __init__(self, name: str):
        self.name = name


class Session:
    def __init__(self, name: str, externalUserId: Optional[str] = None):
        self.name = name
        self.externalUserId = externalUserId


class Agent:
    def __init__(self, name: str):
        self.name = name


class Server:
    def __init__(self, apiKey: str, projectId: str, host: str):
        self.apiKey = apiKey
        self.projectId = projectId
        self.host = host

    def _doReq(self, config: Dict[str, Any]) -> Any:
        headers = config.get('headers', {})
        headers.update({
            'Content-Type': 'application/json',
            'x-observa-api-key': self.apiKey,
            'x-observa-project-id': self.projectId
        })
        try:
            response = requests.request(
                method=config['method'],
                url=config['url'],
                json=config.get('data'),
                headers=headers
            )
            response.raise_for_status()
            return response
        except requests.exceptions.RequestException as error:
            raise Exception(error.response.json().get('message', '未知错误'))

    def createThread(self, thread: Thread) -> str:
        res = self._doReq({
            'url': urljoin(self.host, 'api/v1/thread'),
            'method': 'POST',
            'data': thread.__dict__
        })
        return res.json()['id']

    def createSession(self, session: Session) -> str:
        res = self._doReq({
            'url': urljoin(self.host, 'api/v1/session'),
            'method': 'POST',
            'data': session.__dict__
        })
        return res.json()['id']

    def createAgent(self, agent: Agent) -> str:
        res = self._doReq({
            'url': urljoin(self.host, 'api/v1/agent'),
            'method': 'POST',
            'data': agent.__dict__
        })
        return res.json()['id']

    def createEvent(self, event: Dict[str, Any]) -> None:
        self._doReq({
            'url': urljoin(self.host, 'api/v1/event'),
            'method': 'POST',
            'data': event
        })



from langchain_core.callbacks import BaseCallbackHandler, StdOutCallbackHandler
from langchain_core.outputs import LLMResult
from typing import Dict, Any, List
from langchain_core.messages import BaseMessage
from langchain_google_genai import ChatGoogleGenerativeAI


class LoggingHandler(BaseCallbackHandler):
    def on_chat_model_start(
            self, serialized: Dict[str, Any], messages: List[List[BaseMessage]], **kwargs
    ) -> None:
        print(kwargs.get("run_id"))
        print("Chat model started")

    def on_llm_start(
            self, serialized: Dict[str, Any], prompts: List[str], **kwargs: Any
    ) -> Any:
        """Run when LLM starts running."""

        print("llm model started")

    def on_llm_end(self, response: LLMResult, **kwargs) -> None:
        print(f"Chat model ended, response: {response}")

    def on_chain_start(
            self, serialized: Dict[str, Any], inputs: Dict[str, Any], **kwargs
    ) -> None:
        print(f"Chain {serialized.get('name')} started")

    def on_chain_end(self, outputs: Dict[str, Any], **kwargs) -> None:
        print(f"Chain ended, outputs: {outputs}")


# callbacks = [LoggingHandler()]
#
# llm = ChatGoogleGenerativeAI(
#     model="gemini-2.0-flash-exp",
#     google_api_key="AIzaSyDzj2f8kaoEBFbjY3T0UTtnwTlHRwtvR1Y"
# )
# llm.with_config(callbacks=callbacks)
# res = llm.invoke("给我讲一个笑话")
# print(222, res)

handler = LoggingHandler()
llm = ChatGoogleGenerativeAI(
    model="gemini-2.0-flash-exp",
    google_api_key="AIzaSyDzj2f8kaoEBFbjY3T0UTtnwTlHRwtvR1Y"
)
res = llm.invoke("给我讲一个笑话", {"callbacks": [handler]})
print(222, res)



from setuptools import setup, find_packages

setup(
    name='jackchendongtest',  # 写包的名字
    version='1.1.1',  # Start with a small number and increase it with every change you make
    license='MIT',  # 选择一个开源许可对应你刚才那个from here: https://help.github.com/articles/licensing-a-repository
    description='observa API',  # Give a short description about your library
    author='chendong',  # 填写作者名称
    author_email='chendong@qq.com',  # 填写email
    url='https://github.com',  # Provide either the link to your github or to your website
    packages=find_packages(),
    keywords=['API', 'AI', 'Python'],  # Keywords that define your package best
    install_requires=[  # 包的依赖
        'requests~=2.31.0',  # 可以加上版本号，如validators=1.5.1
        'langchain~=0.3.20',
        'langchain-core~=0.3.45',
    ],
    python_requires='>=3.9',
)



from enum import Enum
from typing import List, Optional, Dict, Any, Union
import time
from langchain_core.outputs import LLMResult
from pydantic import BaseModel


class EventType(str, Enum):
    LLM = 'LLM'
    TOOL = 'TOOL'
    CHAIN = 'CHAIN'
    RETRIEVAL = 'RETRIEVAL'


class Event(BaseModel):
    eventId: str
    eventType: EventType
    eventName: str
    parentId: Optional[str] = None
    children: Optional[List['Event']] = None
    metadata: Optional[Dict] = None
    input: Any = None
    output: Optional[Union[LLMResult, str, int, List[str], Dict[str, Any]]] = None
    startTime: Optional[int] = None
    endTime: Optional[int] = None
    error: Optional[Dict] = None
    tags: Optional[List[str]] = None
    duration: Optional[int] = None
    tokenCount: Optional[int] = None
    extraParams: Optional[Dict] = None
    model: Optional[str] = None


class EventManager:

    def __init__(self, project: str):
        self.project = project
        self.eventMap: Dict[str, Event] = {}
        self.rootEvents: List[Event] = []

    def createEventAndTrace(self, event: Event) -> Event:
        if not event.startTime:
            event.startTime = int(time.time() * 1000)
        if event.tokenCount is None:
            event.tokenCount = 0
        self.addEventToParent(event)
        return event

    def addEventToParent(self, event: Event) -> None:
        if event.parentId:
            parentEvent = self.eventMap.get(event.parentId)
            if parentEvent:
                parentEvent.children.append(event)
        else:
            self.rootEvents.append(event)
        self.eventMap[event.eventId] = event

    def getRootEvents(self) -> List[Dict]:
        return list(map(lambda x: x.model_dump(), self.rootEvents))

    def getEventById(self, eventId: str) -> Optional[Event]:
        return self.eventMap.get(eventId)

    def clear(self) -> None:
        self.rootEvents.clear()
        self.eventMap.clear()
